﻿using System;
using System.Linq;

using ApplicationBoilerplate.DataProvider;
using ApplicationBoilerplate.Events;

using CreativeMinds.StopForumSpam;
using CreativeMinds.StopForumSpam.Responses;

using mvcForum.Core;
using mvcForum.Core.Abstractions.Interfaces;
using mvcForum.Core.Events;
using mvcForum.Core.Interfaces.AddOns;
using mvcForum.Core.Interfaces.Data;
using mvcForum.Core.Interfaces.Events;
using ApplicationBoilerplate.Logging;

namespace mvcForum.AddOns.StopForumSpam {

	public class StopForumSpamAddOn : IAntiSpamAddOn, IAsyncEventListener<NewPostEvent>, IAsyncEventListener<NewTopicEvent>, IAsyncEventListener<PostUpdatedEvent>, IAsyncEventListener<TopicUpdatedEvent>, IEventListener<NewUserEvent> {
		private readonly IConfiguration config;
		private readonly StopForumSpamConfiguration addOnConfig;
		private readonly ILogger logger;
		private readonly IContext context;
		private readonly IRepository<Post> postRepo;
		private readonly IRepository<Topic> topicRepo;
		private readonly IAsyncTask task;

		public StopForumSpamAddOn(StopForumSpamConfiguration addOnConfig, IConfiguration config, ILogger logger, IContext context, IRepository<Post> postRepo, IRepository<Topic> topicRepo, IAsyncTask task) {
			this.config = config;
			this.addOnConfig = addOnConfig;
			this.logger = logger;
			this.context = context;
			this.postRepo = postRepo;
			this.topicRepo = topicRepo;
			this.task = task;
		}

		#region Queue
		public void Queue(TopicUpdatedEvent payload) {
			this.Enqueue(payload);
		}

		public void Queue(PostUpdatedEvent payload) {
			this.Enqueue(payload);
		}

		public void Queue(NewTopicEvent payload) {
			this.Enqueue(payload);
		}

		public void Queue(NewPostEvent payload) {
			this.Enqueue(payload);
		}

		private void Enqueue(Object payload) {
			if (this.addOnConfig.Enabled && this.addOnConfig.MarkAsSpamOnHit) {
				task.Execute(this, payload, this.addOnConfig.Delay);
			}
		}
		#endregion

		#region Handle
		public void Handle(Object payload) {
			if (this.addOnConfig.Enabled && this.addOnConfig.MarkAsSpamOnHit) {
				if (payload is TopicUpdatedEvent) {
					this.Handle((TopicUpdatedEvent)payload);
				}
				else if (payload is PostUpdatedEvent) {
					this.Handle((PostUpdatedEvent)payload);
				}
				else if (payload is NewTopicEvent) {
					this.Handle((NewTopicEvent)payload);
				}
				else if (payload is NewPostEvent) {
					this.Handle((NewPostEvent)payload);
				}
				else {
					throw new ApplicationException("Unknown payload!");
				}
			}
		}

		public void Handle(TopicUpdatedEvent payload) {
			if (this.addOnConfig.Enabled && this.addOnConfig.MarkAsSpamOnHit) {
				Topic topic = this.topicRepo.Read(payload.TopicId);
				this.HandleTopic(topic);
			}
		}

		public void Handle(NewTopicEvent payload) {
			if (this.addOnConfig.Enabled && this.addOnConfig.MarkAsSpamOnHit) {
				Topic topic = this.topicRepo.Read(payload.TopicId);
				this.HandleTopic(topic);
			}
		}

		private void HandleTopic(Topic topic) {
			// Not already deleted or quarantined? Should the updated topic be put in quarantine/marked as spam?
			if ((topic.Flag & TopicFlag.Deleted) == 0 &&
				(topic.Flag & TopicFlag.Quarantined) == 0 &&
				this.IsSpam(topic.Posts.OrderBy(p => p.Posted).First())) {

				if (this.addOnConfig.MarkAsSpamOnHit) {
					topic.SetFlag(TopicFlag.Quarantined);
					// TODO:
					topic.Posts.OrderBy(p => p.Posted).First().EditReason += "StopForumSpam";

					// TODO: Test this null!
					//payload.AddFeedback("StopForumSpam.QuarantineReason", "mvcForum.AddOns", null);
					this.context.SaveChanges();
				}
			}
		}

		public void Handle(PostUpdatedEvent payload) {
			if (this.addOnConfig.Enabled && this.addOnConfig.MarkAsSpamOnHit) {
				Post post = this.postRepo.Read(payload.PostId);
				this.HandlePost(post);
			}
		}

		public void Handle(NewPostEvent payload) {
			if (this.addOnConfig.Enabled && this.addOnConfig.MarkAsSpamOnHit) {
				Post post = this.postRepo.Read(payload.PostId);
				this.HandlePost(post);
			}
		}

		private void HandlePost(Post post) {
			// Not already deleted or quarantined? Should the updated post be put in quarantine/marked as spam?
			if ((post.Flag & PostFlag.Deleted) == 0 &&
				(post.Flag & PostFlag.Quarantined) == 0 &&
				this.IsSpam(post)) {

				if (this.addOnConfig.MarkAsSpamOnHit) {
					post.SetFlag(PostFlag.Quarantined);
					// TODO:
					post.EditReason += "StopForumSpam";

					// TODO: Test this null!
					//payload.AddFeedback("StopForumSpam.QuarantineReason", "mvcForum.AddOns", null);
					this.context.SaveChanges();
				}
			}
		}

		public void Handle(NewUserEvent payload) {
			// Is this add-on enabled? Was the user already identified as a bot?
			if (this.addOnConfig.CheckNewUsers && !payload.Bot) {
				Client client = new Client(this.addOnConfig.Key);
				Response response = client.Check(payload.Username, payload.EmailAddress, payload.IPAddress);
				if (response.Success) {
					payload.Bot = response.ResponseParts.Where(rp => rp.Appears > 0).Any();
				}
			}
		}
		#endregion

		private Boolean IsSpam(Post post) {
			Client client = new Client(this.addOnConfig.Key);
			Response response = client.Check(post.Author.Name, post.Author.EmailAddress, post.IP);
			// TODO: Too simple? Need frequency too? And last seen??
			return response.ResponseParts.Where(rp => rp.Appears > 0).Any();
		}

		public Boolean RunAsynchronously {
			get {
				return this.addOnConfig.RunAsynchronously;
			}
		}

		public Byte Priority {
			get {
				return (Byte)EventPriority.High;
			}
		}

		public Boolean UniqueEvent {
			get {
				return false;
			}
		}
	}
}