﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Xml;

using ApplicationBoilerplate.DataProvider;

using mvcForum.Core.Interfaces.AddOns;
using mvcForum.Core.Specifications;

namespace mvcForum.Core.Abstractions {

	public abstract class AddOnConfiguration<TAddOn> : IAddOnConfiguration<TAddOn> where TAddOn : IAddOn {
		private readonly AddOnConfiguration config;
		private readonly XmlDocument xml;
		private readonly XmlNode root;

		protected AddOnConfiguration(IRepository<AddOnConfiguration> configRepo) {
			String key = this.GetType().Name;
			this.config = configRepo.ReadOne(new AddOnConfigurationSpecifications.SpecificKey(key));
			if (this.config == null) {
				this.config = new AddOnConfiguration(key, "<Settings />");
				configRepo.Create(config);
			}

			this.xml = new XmlDocument();
			this.xml.LoadXml(this.config.Value);
			this.root = xml.SelectSingleNode("./*");
		}

		/// <summary>
		/// Method for getting a collection of ints from the configuration.
		/// </summary>
		/// <param name="key">The key of the configuration.</param>
		/// <returns>The list of ints for the given configuration.</returns>
		protected virtual IEnumerable<Int32> GetCollection(String key) {
			List<Int32> values = new List<Int32>();
			XmlNode keyNode = this.root.SelectSingleNode(key);
			if (keyNode != null) {
				foreach (XmlNode valueNode in keyNode.SelectNodes("Value")) {
					Int32 value;
					if (Int32.TryParse(valueNode.InnerText, out value)) {
						if (!values.Contains(value)) {
							values.Add(value);
						}
					}
				}
			}
			return values;
		}

		/// <summary>
		/// Method for getting a integer value from the configuration.
		/// </summary>
		/// <param name="key">The key of the configuration.</param>
		/// <returns>The integer value from the given configuration.</returns>
		protected virtual Int32 GetInt32(String key) {
			if (this.root.Attributes.GetNamedItem(key) != null) {
				Int32 output;
				if (Int32.TryParse(this.root.Attributes.GetNamedItem(key).Value, out output)) {
					return output;
				}
			}
			this.Set(key, 0);
			return this.GetInt32(key);
		}

		protected virtual String GetString(String key) {
			if (this.root.SelectSingleNode(key) != null) {
				return this.root.SelectSingleNode(key).InnerText;
			}
			this.Set(key, "");
			return this.GetString(key);
		}

		protected virtual Boolean GetBoolean(String key) {
			if (this.root.Attributes.GetNamedItem(key) != null) {
				Boolean output;
				if (Boolean.TryParse(this.root.Attributes.GetNamedItem(key).Value, out output)) {
					return output;
				}
			}
			this.Set(key, false);
			return this.GetBoolean(key);
		}

		protected virtual void Set(String key, IEnumerable<Int32> values) {
			XmlNode keyNode = this.root.SelectSingleNode(key);
			if (keyNode == null) {
				keyNode = xml.CreateElement(key);
				this.root.AppendChild(keyNode);
			}
			keyNode.RemoveAll();

			foreach (Int32 value in values) {
				XmlNode valueNode = xml.CreateElement("Value");
				keyNode.AppendChild(valueNode);
				valueNode.AppendChild(xml.CreateCDataSection(value.ToString()));
			}
			this.config.Value = this.xml.OuterXml;
		}
		protected virtual void Set(String key, Int32 value) {
			this.SetAttributeValue(key, value.ToString());
		}
		protected virtual void Set(String key, Boolean value) {
			this.SetAttributeValue(key, value.ToString());
		}
		protected virtual void Set(String key, String value) {
			this.SetNodeContent(key, value);
		}

		private void SetAttributeValue(String key, String value) {
			if (this.root.Attributes.GetNamedItem(key) == null) {
				this.root.Attributes.Append(xml.CreateAttribute(key));
			}
			this.root.Attributes.GetNamedItem(key).Value = value;
			this.config.Value = this.xml.OuterXml;
		}

		private void SetNodeContent(String key, String value) {
			if (this.root.SelectSingleNode(key) == null) {
				XmlNode keyNode = xml.CreateElement(key);
				this.root.AppendChild(keyNode);
			}

			this.root.SelectSingleNode(key).InnerText = value;
			this.config.Value = this.xml.OuterXml;
		}
	}
}