﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Configuration.Provider;
using System.Linq;
using System.Security.Cryptography;
using System.Text;
using System.Web.Hosting;
using System.Web.Security;
using MongoDB.Bson;
using MongoDB.Driver;
using MongoDB.Driver.Builders;

namespace mvcForum.MembershipProviderTest.Security {

	public class MongoDBMembershipProvider : MembershipProvider {
		private Boolean enablePasswordReset;
		private Boolean enablePasswordRetrieval;
		private Int32 maxInvalidPasswordAttempts;
		private Int32 minRequiredNonAlphanumericCharacters;
		private Int32 minRequiredPasswordLength;
		private Int32 passwordAttemptWindow;
		private MembershipPasswordFormat passwordFormat;
		private String passwordStrengthRegularExpression;
		private Boolean requiresUniqueEmail;
		private MongoConnection conn;

		private MongoConnection Connection {
			get {
				if (this.conn == null) {
					this.conn = MongoConnection.Create("User.MongoDB");
				}
				return this.conn;
			}
		}

		protected String EncodePassword(String password, MembershipPasswordFormat membershipPasswordFormat, String salt) {
			if (String.IsNullOrWhiteSpace(password)) {
				return null;
			}

			if (membershipPasswordFormat == MembershipPasswordFormat.Clear) {
				return password;
			}

			Byte[] passwordBytes = Encoding.Unicode.GetBytes(password);
			Byte[] saltBytes = Convert.FromBase64String(salt);
			Byte[] allBytes = new Byte[saltBytes.Length + passwordBytes.Length];

			Buffer.BlockCopy(saltBytes, 0, allBytes, 0, saltBytes.Length);
			Buffer.BlockCopy(passwordBytes, 0, allBytes, saltBytes.Length, passwordBytes.Length);

			if (membershipPasswordFormat == MembershipPasswordFormat.Hashed) {
				return Convert.ToBase64String(HashAlgorithm.Create("SHA1").ComputeHash(allBytes));
			}

			return Convert.ToBase64String(this.EncryptPassword(allBytes));
		}

		protected void ValidateNewPassword(String username, String password) {
			if (password.Length < this.MinRequiredPasswordLength) {
				throw new MembershipPasswordException("too short");
			}
			Int32 nonAlphaNumeric = 0;
			for (Int32 index = 0; index < password.Length; index++) {
				if (!Char.IsLetterOrDigit(password, index)) {
					nonAlphaNumeric++;
				}
			}
			if (nonAlphaNumeric < this.MinRequiredNonAlphanumericCharacters) {
				throw new MembershipPasswordException("too few non alphanumeric characters");
			}
			//if (base.PasswordStrengthRegularExpression.Length > 0 && !Regex.IsMatch(newPassword, base.PasswordStrengthRegularExpression)) {
			//    throw new ArgumentException();
			//}

			ValidatePasswordEventArgs validatePasswordEventArgs = new ValidatePasswordEventArgs(username, password, false);
			OnValidatingPassword(validatePasswordEventArgs);

			if (validatePasswordEventArgs.Cancel) {
				throw new MembershipPasswordException(validatePasswordEventArgs.FailureInformation.Message);
			}
		}

		protected Boolean VerifyPassword(User user, String password) {
			return user.Password == this.EncodePassword(password, this.PasswordFormat, user.Salt);
		}

		protected MembershipUser ToMembershipUser(User user) {
			if (user == null) {
				return null;
			}

			String email = user.Email ?? null;

			return new MembershipUser(this.Name, user.Username, user.Id, email, null, null, user.IsApproved, user.IsLockedOut, user.CreationDate, user.LastLoginDate, user.LastActivityDate, user.LastPasswordChangedDate, user.LastLockoutDate);
		}

		protected String DecodePassword(String password, MembershipPasswordFormat membershipPasswordFormat) {
			switch (passwordFormat) {
				case MembershipPasswordFormat.Clear:
					return password;

				case MembershipPasswordFormat.Hashed:
					throw new ProviderException("Hashed passwords cannot be decoded.");

				default:
					Byte[] passwordBytes = Convert.FromBase64String(password);
					Byte[] decryptedBytes = this.DecryptPassword(passwordBytes);
					return decryptedBytes == null ? null : Encoding.Unicode.GetString(decryptedBytes, 16, decryptedBytes.Length - 16);
			}
		}

		#region Public Methods
		public override Boolean ChangePassword(String username, String oldPassword, String newPassword) {
			User user = this.GetUserByName(username);

			if (!this.VerifyPassword(user, oldPassword)) {
				return false;
			}

			this.ValidateNewPassword(username, newPassword);

			user.LastPasswordChangedDate = DateTime.UtcNow;
			user.Password = this.EncodePassword(newPassword, this.PasswordFormat, user.Salt);
			return this.UpdateUser(user);
		}

		public override Boolean ChangePasswordQuestionAndAnswer(String username, String password, String newPasswordQuestion, String newPasswordAnswer) {
			return false;
		}

		public override MembershipUser CreateUser(String username, String password, String email, String passwordQuestion, String passwordAnswer, Boolean isApproved, Object providerUserKey, out MembershipCreateStatus status) {
			if (providerUserKey != null) {
				if (!(providerUserKey is ObjectId)) {
					status = MembershipCreateStatus.InvalidProviderUserKey;
					return null;
				}
			}
			else {
				providerUserKey = ObjectId.GenerateNewId();
			}

			try {
				this.ValidateNewPassword(username, password);
			}
			catch (MembershipPasswordException) {
				status = MembershipCreateStatus.InvalidPassword;
				return null;
			}

			if (this.GetUser(username, false) != null) {
				status = MembershipCreateStatus.DuplicateUserName;
				return null;
			}

			if (this.GetUser(providerUserKey, false) != null) {
				status = MembershipCreateStatus.DuplicateProviderUserKey;
				return null;
			}

			if (this.RequiresUniqueEmail && !String.IsNullOrWhiteSpace(this.GetUserNameByEmail(email))) {
				status = MembershipCreateStatus.DuplicateEmail;
				return null;
			}

			Byte[] buffer = new Byte[16];
			(new RNGCryptoServiceProvider()).GetBytes(buffer);
			String salt = Convert.ToBase64String(buffer);

			DateTime creationDate = DateTime.UtcNow;

			this.CreateUser(
					(ObjectId)providerUserKey,
					creationDate,
					email,
					isApproved,
					password,
					passwordAnswer,
					passwordQuestion,
					salt,
					username
				);

			status = MembershipCreateStatus.Success;
			return this.GetUser(username, false);
		}

		public override Boolean DeleteUser(String username, Boolean deleteAllRelatedData) {
			return this.DeleteUserInternal(username, deleteAllRelatedData);
		}

		public override MembershipUserCollection FindUsersByEmail(String emailToMatch, Int32 pageIndex, Int32 pageSize, out Int32 totalRecords) {
			MembershipUserCollection membershipUsers = new MembershipUserCollection();

			IEnumerable<User> users = this.GetUsersByEmail(emailToMatch);
			totalRecords = users.Count();

			foreach (User user in users.Skip(pageIndex * pageSize).Take(pageSize)) {
				membershipUsers.Add(this.ToMembershipUser(user));
			}

			return membershipUsers;
		}

		public override MembershipUserCollection FindUsersByName(String usernameToMatch, Int32 pageIndex, Int32 pageSize, out Int32 totalRecords) {
			MembershipUserCollection membershipUsers = new MembershipUserCollection();

			IEnumerable<User> users = this.GetUsersByName(usernameToMatch);
			totalRecords = users.Count();

			foreach (User user in users.Skip(pageIndex * pageSize).Take(pageSize)) {
				membershipUsers.Add(ToMembershipUser(user));
			}

			return membershipUsers;
		}

		public override MembershipUserCollection GetAllUsers(Int32 pageIndex, Int32 pageSize, out Int32 totalRecords) {
			MembershipUserCollection membershipUsers = new MembershipUserCollection();

			IEnumerable<User> users = this.GetAllUsersInternal();
			totalRecords = users.Count();

			foreach (User user in users.Skip(pageIndex * pageSize).Take(pageSize)) {
				membershipUsers.Add(this.ToMembershipUser(user));
			}

			return membershipUsers;
		}

		public override Int32 GetNumberOfUsersOnline() {
			return this.GetOnlineUsers();
		}

		public override String GetPassword(String username, String answer) {
			//if (!this.EnablePasswordRetrieval) {
			throw new NotSupportedException("This Membership Provider has not been configured to support password retrieval.");
			//}

			//User user = this.GetUserByName(username);

			////if (this.RequiresQuestionAndAnswer && !this.VerifyPasswordAnswer(user, answer)) {
			////	throw new MembershipPasswordException("The password-answer supplied is invalid.");
			////}

			//return this.DecodePassword(user.PasswordQuestion, this.PasswordFormat);
		}

		public override MembershipUser GetUser(String username, Boolean userIsOnline) {
			User user = this.GetUserByName(username);
			if (user == null) {
				return null;
			}

			if (userIsOnline == true) {
				user.LastActivityDate = DateTime.UtcNow;
				this.UpdateUser(user);
			}

			return this.ToMembershipUser(user);
		}

		public override MembershipUser GetUser(Object providerUserKey, Boolean userIsOnline) {
			User user = null;
			if (providerUserKey is String) {
				user = this.GetUserByProviderId(ObjectId.Parse((String)providerUserKey));
			}
			else {
				user = this.GetUserByProviderId((ObjectId)providerUserKey);
			}

			if (user == null) {
				return null;
			}

			if (userIsOnline == true) {
				user.LastActivityDate = DateTime.UtcNow;
				this.UpdateUser(user);
			}

			return this.ToMembershipUser(user);
		}

		public override String GetUserNameByEmail(String email) {
			User user = this.GetUserByEmail(email);
			return user != null ? user.Username : null;
		}

		public override void Initialize(String name, NameValueCollection config) {
			this.ApplicationName = config["applicationName"] ?? HostingEnvironment.ApplicationVirtualPath ?? String.Empty;
			this.enablePasswordReset = Boolean.Parse(config["enablePasswordReset"] ?? "true");
			this.enablePasswordRetrieval = Boolean.Parse(config["enablePasswordRetrieval"] ?? "false");
			this.maxInvalidPasswordAttempts = Int32.Parse(config["maxInvalidPasswordAttempts"] ?? "5");
			this.minRequiredNonAlphanumericCharacters = Int32.Parse(config["minRequiredNonAlphanumericCharacters"] ?? "1");
			this.minRequiredPasswordLength = Int32.Parse(config["minRequiredPasswordLength"] ?? "7");
			this.passwordAttemptWindow = Int32.Parse(config["passwordAttemptWindow"] ?? "10");
			this.passwordFormat = (MembershipPasswordFormat)Enum.Parse(typeof(MembershipPasswordFormat), config["passwordFormat"] ?? "Hashed");
			this.passwordStrengthRegularExpression = config["passwordStrengthRegularExpression"] ?? String.Empty;
			this.requiresUniqueEmail = Boolean.Parse(config["requiresUniqueEmail"] ?? "true");

			if (this.PasswordFormat == MembershipPasswordFormat.Hashed && this.EnablePasswordRetrieval) {
				throw new ProviderException("Configured settings are invalid: Hashed passwords cannot be retrieved. Either set the password format to different type, or set enablePasswordRetrieval to false.");
			}

			base.Initialize(name, config);
		}

		public override String ResetPassword(String username, String answer) {
			if (!this.EnablePasswordReset) {
				throw new NotSupportedException("This provider is not configured to allow password resets. To enable password reset, set enablePasswordReset to \"true\" in the configuration file.");
			}

			User user = this.GetUserByName(username);

			String password = Membership.GeneratePassword(this.MinRequiredPasswordLength, this.MinRequiredNonAlphanumericCharacters);
			user.LastPasswordChangedDate = DateTime.UtcNow;
			user.Password = this.EncodePassword(password, this.PasswordFormat, user.Salt);
			this.UpdateUser(user);

			return password;
		}

		public override Boolean UnlockUser(String username) {
			User user = this.GetUserByName(username);

			user.FailedPasswordAttemptCount = 0;
			user.FailedPasswordAttemptWindowStart = new DateTime(1970, 1, 1);
			user.IsLockedOut = false;
			user.LastLockoutDate = new DateTime(1970, 1, 1);
			this.UpdateUser(user);

			return true;
		}

		public override void UpdateUser(MembershipUser user) {
			User u = this.GetUserByProviderId((ObjectId)user.ProviderUserKey);

			if (u == null) {
				throw new ProviderException("The user was not found.");
			}

			u.Email = user.Email;
			u.IsApproved = user.IsApproved;
			u.LastActivityDate = user.LastActivityDate.ToUniversalTime();
			u.LastLoginDate = user.LastLockoutDate.ToUniversalTime();

			this.UpdateUser(u);
		}

		public override Boolean ValidateUser(String username, String password) {
			User user = this.GetUserByName(username);

			// No user, or user is not approved, or user is locked out, or account is an external account!
			if (user == null || !user.IsApproved || user.IsLockedOut /*|| user.Identity != null*/) {
				return false;
			}

			if (this.VerifyPassword(user, password)) {
				user.LastLoginDate = user.LastActivityDate = DateTime.UtcNow;
				this.UpdateUser(user);
				return true;
			}

			Int32 attemptCount = user.FailedPasswordAttemptCount + 1;
			// Outside the fail windows??
			if (DateTime.UtcNow > user.FailedPasswordAttemptWindowStart.AddMinutes(this.PasswordAttemptWindow)) {
				// Let's set the attemt count to 1 and restart the window!!
				user.FailedPasswordAttemptCount = 1;
				user.FailedPasswordAttemptWindowStart = DateTime.UtcNow;
			}
			else {
				// Too many attempts??
				if (attemptCount >= this.MaxInvalidPasswordAttempts) {
					// Let's lock-out the user!!
					user.IsLockedOut = true;
					user.LastLockoutDate = DateTime.UtcNow;
				}
				else {
					// No, let's add to the count!
					user.FailedPasswordAttemptCount = attemptCount;
				}
			}
			this.UpdateUser(user);

			return false;
		}
		#endregion

		private IMongoQuery BuildQuery(params IMongoQuery[] query) {
			return Query.And(new IMongoQuery[] { this.BuildApplicationQuery() }.Union(query).ToArray());
		}

		private IMongoQuery BuildApplicationQuery() {
			return Query.EQ(User.FieldNames.ApplicationName, this.ApplicationName);
		}

		protected IEnumerable<User> GetUsersByName(String name) {
			return this.Connection
					.GetCollection<User>()
					.FindAs<User>(this.BuildQuery(new IMongoQuery[] { Query.Matches(User.FieldNames.UpperUsername, name.ToUpperInvariant()) }))
					.ToList();
		}

		#region Properties
		public override String ApplicationName { get; set; }

		public override Boolean EnablePasswordReset {
			get {
				return this.enablePasswordReset;
			}
		}

		public override Boolean EnablePasswordRetrieval {
			get {
				return this.enablePasswordRetrieval;
			}
		}

		public override Int32 MaxInvalidPasswordAttempts {
			get {
				return this.maxInvalidPasswordAttempts;
			}
		}

		public override Int32 MinRequiredNonAlphanumericCharacters {
			get {
				return this.minRequiredNonAlphanumericCharacters;
			}
		}

		public override Int32 MinRequiredPasswordLength {
			get {
				return this.minRequiredPasswordLength;
			}
		}

		public override Int32 PasswordAttemptWindow {
			get {
				return this.passwordAttemptWindow;
			}
		}

		public override MembershipPasswordFormat PasswordFormat {
			get {
				return this.passwordFormat;
			}
		}

		public override String PasswordStrengthRegularExpression {
			get {
				return this.passwordStrengthRegularExpression;
			}
		}

		public override Boolean RequiresQuestionAndAnswer {
			get {
				return false;
			}
		}

		public override Boolean RequiresUniqueEmail {
			get {
				return this.requiresUniqueEmail;
			}
		}
		#endregion

		protected IEnumerable<User> GetUsersByEmail(String email) {
			return this.Connection
					.GetCollection<User>()
					.FindAs<User>(this.BuildQuery(new IMongoQuery[] { Query.Matches(User.FieldNames.Email, email.ToLowerInvariant()) }))
					.ToList();
		}

		protected Int32 GetOnlineUsers() {
			TimeSpan timeSpan = TimeSpan.FromMinutes(Membership.UserIsOnlineTimeWindow);
			return (Int32)this.Connection
					.GetCollection<User>()
					.FindAs<User>(this.BuildQuery(Query.GT(User.FieldNames.LastActivityDate, DateTime.UtcNow.Subtract(timeSpan))))
					.Count();
		}

		protected IEnumerable<User> GetAllUsersInternal() {
			return this.Connection
					.GetCollection<User>()
					.FindAs<User>(this.BuildApplicationQuery()).ToList();
		}

		protected User GetUserByEmail(String email) {
			return this.Connection
					.GetCollection<User>()
					.FindOneAs<User>(this.BuildQuery(Query.EQ(User.FieldNames.Email, email.ToLowerInvariant())));
		}

		protected User GetUserByName(String username) {
			return this.Connection
					.GetCollection<User>()
					.FindOneAs<User>(this.BuildQuery(Query.EQ(User.FieldNames.UpperUsername, username.ToUpperInvariant())));
		}

		protected User GetUserByProviderId(ObjectId id) {
			return this.Connection
					.GetCollection<User>()
					.FindOneByIdAs<User>(id);
		}

		protected Boolean UpdateUser(User user) {
			User u = user as User;
			if (u == null) {
				throw new ArgumentException("user");
			}
			// Make sure the e-mail address is in lower-case!
			u.Email = user.Email.ToLowerInvariant();
			// Make sure the username is stored in upper-case too, so we can search after it!
			u.UpperUsername = user.Username.ToUpperInvariant();
			return this.Connection.GetCollection<User>().Save<User>(u).Ok;
		}

		protected void CreateUser(ObjectId id, DateTime creationDate, String emailAddress, Boolean isApproved, String password, String passwordAnswer, String passwordQuestion, String salt, String username) {
			User user = new User {
				Id = id,
				ApplicationName = this.ApplicationName,
				CreationDate = creationDate,
				Email = emailAddress.ToLowerInvariant(),
				FailedPasswordAttemptWindowStart = creationDate,
				FailedPasswordAttemptCount = 0,
				IsApproved = isApproved,
				IsLockedOut = false,
				LastActivityDate = creationDate,
				LastLockoutDate = new DateTime(1970, 1, 1),
				LastLoginDate = creationDate,
				LastPasswordChangedDate = creationDate,
				Password = this.EncodePassword(password, this.PasswordFormat, salt),
				Salt = salt,
				Username = username,
				UpperUsername = username.ToUpperInvariant()
			};

			this.Connection.GetCollection<User>().Insert<User>(user);
		}

		protected Boolean DeleteUserInternal(String username, Boolean deleteAllRelatedData) {
			return this.Connection
					.GetCollection<User>()
					.Remove(this.BuildQuery(Query.EQ(User.FieldNames.UpperUsername, username.ToUpperInvariant())))
					.Ok;
		}
	}
}
