﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Web.Mvc;
using System.Web.Routing;

using MVCBootstrap.Web.Mvc.Navigations;
using SimpleLocalisation;

namespace mvcForum.Web.Extensions {

	public static class HtmlExtensions {

		public static IEnumerable<NavigationItem> Navigation(this HtmlHelper html, UrlHelper url, String name) {
			// Get the navigation with the matching name!
			INavigation navigation = DependencyResolver.Current.GetServices<INavigation>().Where(n => n.Name == name).FirstOrDefault();
			// Found one?
			if (navigation == null) {
				// No navigation found, let's "fail" gracefully!
				return new List<NavigationItem>();
			}

			// Has the navigation been initialized?
			if (!navigation.Initialized) {
				// No, let's do it!
				navigation.Initialize(url);
			}

			// Return the list of navigation items!
			return navigation.GetNavigation(html);
		}

		public static MvcHtmlString Link(this HtmlHelper html, String title, String url, Object htmlAttributes) {
			TagBuilder tag = new TagBuilder("a");
			tag.MergeAttribute("href", url);
			tag.SetInnerText(title);
			RouteValueDictionary attr = HtmlHelper.AnonymousObjectToHtmlAttributes(htmlAttributes);
			tag.MergeAttributes<String, Object>(attr);

			return tag.ToMvcHtmlString();
		}

		public static String IdFor<TModel, TValue>(this HtmlHelper<TModel> html, Expression<Func<TModel, TValue>> expression) {
			ModelMetadata metadata = ModelMetadata.FromLambdaExpression<TModel, TValue>(expression, html.ViewData);
			String fieldname = ExpressionHelper.GetExpressionText(expression);
			fieldname = metadata.DisplayName ?? metadata.PropertyName ?? fieldname.Split(new Char[] { '.' }).Last<String>();

			return TagBuilder.CreateSanitizedId(html.ViewContext.ViewData.TemplateInfo.GetFullHtmlFieldName(fieldname));
		}

		public static String DisplayNameFor<TModel, TValue>(this HtmlHelper<TModel> html, Expression<Func<TModel, TValue>> expression) {
			ModelMetadata metadata = ModelMetadata.FromLambdaExpression<TModel, TValue>(expression, html.ViewData);
			String fieldname = ExpressionHelper.GetExpressionText(expression);

			fieldname = metadata.DisplayName ?? metadata.PropertyName ?? fieldname.Split(new Char[] { '.' }).Last<String>();
			if (String.IsNullOrEmpty(fieldname)) {
				return String.Empty;
			}
			return fieldname;
		}

		/// <summary>
		/// Get the localized string with the given key.
		/// </summary>
		/// <param name="html">The HtmlHelper.</param>
		/// <param name="key">The key of the localized string.</param>
		/// <returns>A localized string.</returns>
		public static String LocalizedString(this HtmlHelper html, String key) {
			return html.LocalizedString(key, String.Empty, null);
		}

		/// <summary>
		/// Get the localized string with the given key and namespace.
		/// </summary>
		/// <param name="html">The HtmlHelper.</param>
		/// <param name="key">The key of the localized string.</param>
		/// <param name="namespace">The namespace.</param>
		/// <returns>A localized string.</returns>
		public static String LocalizedString(this HtmlHelper html, String key, String @namespace) {
			return html.LocalizedString(key, @namespace, null);
		}

		/// <summary>
		/// Get the localized string with the given key, namespace and values.
		/// </summary>
		/// <param name="html">The HtmlHelper.</param>
		/// <param name="key">The key of the localized string.</param>
		/// <param name="namespace">The namespace.</param>
		/// <param name="values">The values to insert into the string.</param>
		/// <returns>A localized string.</returns>
		public static String LocalizedString(this HtmlHelper html, String key, String @namespace, Object values) {
			TextManager text = DependencyResolver.Current.GetService<TextManager>();
			if (text != null) {
				return text.Get(key, values: values, ns: @namespace);
			}
			return String.Empty;
		}

		public static String LocalizedString<TNamespace>(this HtmlHelper html, String key) {
			return html.LocalizedString<TNamespace>(key, null);
		}

		public static String LocalizedString<TNamespace>(this HtmlHelper html, String key, Object values) {
			TextManager text = DependencyResolver.Current.GetService<TextManager>();
			if (text != null) {
				return text.Get<TNamespace>(key, values: values);
			}
			return String.Empty;
		}

		/// <summary>
		/// Get the localized html string with the given key.
		/// </summary>
		/// <param name="html">The HtmlHelper.</param>
		/// <param name="key">The key of the localized html string.</param>
		/// <returns>A localized html string.</returns>
		public static MvcHtmlString LocalizedHtmlString(this HtmlHelper html, String key) {
			return MvcHtmlString.Create(html.LocalizedString(key));
		}

		/// <summary>
		/// Get the localized html string with the given key and namespace.
		/// </summary>
		/// <param name="html">The HtmlHelper.</param>
		/// <param name="key">The key of the localized html string.</param>
		/// <param name="namespace">The namespace.</param>
		/// <returns>A localized html string.</returns>
		public static MvcHtmlString LocalizedHtmlString(this HtmlHelper html, String key, String @namespace) {
			return MvcHtmlString.Create(html.LocalizedString(key, @namespace));
		}

		/// <summary>
		/// Get the localized html string with the given key, namespace and values.
		/// </summary>
		/// <param name="html">The HtmlHelper.</param>
		/// <param name="key">The key of the localized html string.</param>
		/// <param name="namespace">The namespace.</param>
		/// <param name="values">The values to insert into the string.</param>
		/// <returns>A localized html string.</returns>
		public static MvcHtmlString LocalizedHtmlString(this HtmlHelper html, String key, String @namespace, Object values) {
			return MvcHtmlString.Create(html.LocalizedString(key, @namespace, values));
		}
	}
}